/*
** File: coremisc.c  
** Project: ADB3 core driver
** Purpose: Miscellaneous Core Interface functions.
**
** (C) Copyright Alpha Data 2009-2010
*/

#include <df.h>
#include "device.h"
#include "coremisc.h"

unsigned int
coreGetNumWindow(
  void* pInterfaceContext)
{
  Adb3CoreDeviceContext* pDevCtx = (Adb3CoreDeviceContext*)pInterfaceContext;

  return pDevCtx->info.bootstrap.numWindow;
}

CoreWindowStatus
coreGetWindowInfo(
  void* pInterfaceContext,
  unsigned int windowIndex,
  CoreWindowInfo* pInfo)
{
  Adb3CoreDeviceContext* pDevCtx = (Adb3CoreDeviceContext*)pInterfaceContext;
  WindowInfo* pWindowInfo;

  if (windowIndex >= pDevCtx->info.bootstrap.numWindow) {
    return CoreWindowInvalidIndex;
  } else {
    pWindowInfo = &pDevCtx->info.window[windowIndex];
    pInfo->busBase = pWindowInfo->busBase;
    pInfo->busSize = pWindowInfo->busSize;
    pInfo->localBase = pWindowInfo->localBase;
    pInfo->localSize = pWindowInfo->localSize;
    pInfo->translatedBase = pWindowInfo->translatedBase;
    pInfo->translatedSize = pWindowInfo->translatedSize;
    pInfo->pKernelBase = pWindowInfo->pKernelBase;
    pInfo->kernelSize = pWindowInfo->kernelSize;
    return CoreWindowSuccess;
  }
}

CoreWindowAddressStatus
coreGetWindowAddress(
  void* pInterfaceContext,
  unsigned int windowIndex,
  size_t offset,
  size_t length,
  void** pKernelAddress)
{
  Adb3CoreDeviceContext* pDevCtx = (Adb3CoreDeviceContext*)pInterfaceContext;
  WindowInfo* pWindowInfo;

  if (windowIndex >= pDevCtx->info.bootstrap.numWindow) {
    return CoreWindowAddressInvalidIndex;
  } else {
    pWindowInfo = &pDevCtx->info.window[windowIndex];
    if (offset >= pWindowInfo->kernelSize || length > pWindowInfo->kernelSize || offset + length > pWindowInfo->kernelSize) {
      return CoreWindowAddressInvalidRegion;
    }
    *pKernelAddress = (uint8_t*)pWindowInfo->pKernelBase + offset;
    return CoreWindowAddressSuccess;
  }
}

unsigned int
coreGetNumDmaChannel(
  void* pInterfaceContext)
{
  Adb3CoreDeviceContext* pDevCtx = (Adb3CoreDeviceContext*)pInterfaceContext;

  return pDevCtx->info.bootstrap.numDmaChannel;
}

unsigned int
coreGetNumCommonBuffer(
  void* pInterfaceContext)
{
  Adb3CoreDeviceContext* pDevCtx = (Adb3CoreDeviceContext*)pInterfaceContext;

  return pDevCtx->comBuf.count;
}

CoreCommonBufferStatus
coreGetCommonBufferInfo(
  void* pInterfaceContext,
  unsigned int commonBufferIndex,
  CoreCommonBufferInfo* pInfo)
{
  Adb3CoreDeviceContext* pDevCtx = (Adb3CoreDeviceContext*)pInterfaceContext;
  DfDmaBuffer* pDmaBuffer;

  if (commonBufferIndex >= pDevCtx->comBuf.count) {
    return CoreCommonBufferInvalidIndex;
  } else {
    pDmaBuffer = pDevCtx->comBuf.pComBufs[commonBufferIndex];
    pInfo->busBase = pDmaBuffer->busAddress;
    pInfo->pKernelBase = pDmaBuffer->pKernelAddress;
    pInfo->size = pDmaBuffer->size;
    return CoreWindowSuccess;
  }
}
